/**
 * jQuery Accordion Plugin by iThemes
*/
;(function ( $, window, document, undefined ) {
	// Create the defaults once
	var pluginName = 'iThemesAccordion';

	// The actual plugin constructor
	function Plugin( element ) {

		// Set element
		this.element = element;

		// Set options and Merge defaults with options
		if ( itAccordionGroupData && itAccordionGroupData[element.id] ) {
			this.options = itAccordionGroupData[element.id];
		} else {
			this.options = {};
		}

		this._name = pluginName;

		this.init();
	}

	Plugin.prototype.init = function () {
		// Place initialization logic here
		// You already have access to the DOM element and
		// the options via the instance, e.g. this.element 
		// and this.options
		switch( this.options.theme ) {
			case 'accordion-horizontal-1' :
			case 'accordion-horizontal-2' :
				this.initLegacyHorizontal();
				break;
			case 'accordion-vertical-1' :
			case 'accordion-vertical-2' :
			default :
				this.initLegacyVertical();
				break;
		}
	};

	/**
	 * This method controls the JS for Vertical 1 and Vertical 2 themes
	 * It relies on jquery.tools
	*/
	Plugin.prototype.initLegacyVertical = function() {
		var numh2s = $('#' + this.element.id + ' > div > .accordion-item').length;

		// Set some function vars to pass to addEffect
		var initialIndex = this.options.selected_index - 1;
		if ( initialIndex > (numh2s - 1) ) {
			initialIndex = numh2s-1;
		} else if ( initialIndex < 0 ) {
			initialIndex = -1;
		}
		var slide_up = parseInt(this.options.accordion_slide_up),
			slide_down = parseInt(this.options.accordion_slide_down),
		    tabsOptions = {
				tabs: 'h2.accordion-item',
				effect: 'slide',
				initialIndex: parseInt(initialIndex),
			};

			tabsDiv = $('#' + this.element.id + ' .' + this.options.theme + ' div.pane' );

		// Add event mouseover if set.
		if ( 'on' == this.options.hover_items ) {
			tabsOptions.event = 'mouseover';
		}
		
		// Bind Accordion effects
		$(this.element).tabs(tabsDiv, tabsOptions );

		$.tools.tabs.addEffect("slide", function(i, done) {
			this.getPanes().slideUp( slide_up );
			this.getPanes().eq(i).slideDown( slide_down, function()  {
				$(this).css({backgroundColor: 'transparent'});
				done.call();
			});     
		}); //end addEffect slide
		
	}

	/**
	 * This method controls the JS for Horizontal 1 and Horizontal 2 themes
	 * It relies in jquery.tools
	*/
	Plugin.prototype.initLegacyHorizontal = function() {

		// Position all titles in based tabs 10px from top of tab
		$('.accordion_vertical_text_container h2 span', this.element ).each(function(){
			var textOffset = $(this).innerWidth() + 10;
			$(this).css(
				{
					'transform':'rotate(270deg) translate(-' + textOffset + 'px,0)',
					'-webkit-transform':'rotate(270deg) translate(-' + textOffset + 'px,0)',
					'-moz-transform':'rotate(270deg) translate(-' + textOffset + 'px,0)',
					'-ms-transform':'rotate(270deg) translate(-' + textOffset + 'px,0)',
					'-o-transform':'rotate(270deg) translate(-' + textOffset + 'px,0)'
				}
			);
		});

		// Make all tabs the same width
		$('h2.horizontal-tab', this.element).css('width',this.options.thumbnail_width + 'px');

		var h2TotalWidths = 0;
		$('h2.horizontal-tab', this.element ).each(function(){
			h2TotalWidths += $(this).outerWidth();
		});
		var containerHeight = $('.accordion', this.element).height();
		var containerWidth = $('.accordion', this.element).innerWidth();
		var numh2s = $('div .accordion-item', this.element).length;
		var contentOuterWidth = $('.pane', this.element).outerWidth();
		var contentWidth = $('.pane', this.element).width();
		var contentWidthDiff = contentOuterWidth - contentWidth;
		this.options.contentWidth = containerWidth - h2TotalWidths;

		// Set content width
		contentWidth = containerWidth - h2TotalWidths - contentWidthDiff;

		// Set the vertical text containers to the same height as the accordion div
		$('h2.horizontal-tab', this.element ).css('height', containerHeight+'px');

		// Set some function vars to pass to addEffect
		var initialIndex = this.options.selected_index - 1;
		if ( initialIndex > (numh2s - 1) ) {
			initialIndex = numh2s-1;
		} else if ( initialIndex < 0 ) {
			initialIndex = 0;
		}

		$('.accordion-item .pane', this.element).css({'max-width': contentWidth + 'px', 'width': contentWidth + 'px'});

		var slide_left = parseInt(this.options.accordion_slide_left),
		    slide_right = parseInt(this.options.accordion_slide_right),
		    tabsOptions = {
				tabs: 'h2.horizontal-tab',
				effect: 'horizontal',
				initialIndex: parseInt(initialIndex)
			},
			tabsDiv = $('.' + this.options.theme + ' .accordion-item .pane', this.element );

		// Add event mouseover if set.
		if ( 'on' == this.options.hover_items ) {
			tabsOptions.event = 'mouseover';
		}
		
		// Bind Accordion effects
		$(this.element).tabs( tabsDiv, tabsOptions );

		// Add Horizontal effect
		$.tools.tabs.addEffect("horizontal", function(i, done) {

			var $currentPane = $(this.getCurrentPane());

			// Calculate tabwidth
			if ( $currentPane.data('tabwidth') ) {
				var tabwidth = $currentPane.data('tabwidth');
			} else {
				var tabwidth = $currentPane.width();
			}
			if ( tabwidth ) {
				$currentPane.data('tabwidth', tabwidth);
			}

			// Don't fire if we're currently animating
			if ( ! $currentPane.data('animating') ) {
				$currentPane.data('animating', true);
				var pane = this;
				pane.getPanes().children().css({'max-width': tabwidth,'min-width': tabwidth});
				pane.getCurrentPane().animate( {'width': 0}, slide_left, 'linear', function() {
					$(this).hide();
					pane.getPanes().eq( i ).css({'max-width': tabwidth, 'width':0}).show().animate( { width: tabwidth }, slide_right, 'linear', function() {
						done.call();
						$currentPane.data('animating', false);
					});
				});
			}
		});
	}

	// A really lightweight plugin wrapper around the constructor, 
	// preventing against multiple instantiations
	$.fn[pluginName] = function () {
		return this.each(function () {
			if (!$.data(this, 'plugin_' + pluginName)) {
				$.data(this, 'plugin_' + pluginName, 
				new Plugin( this ));
			}
		});
	}
})( jQuery, window, document );
